<?php return array(
    'root' => array(
        'pretty_version' => 'dev-main',
        'version' => 'dev-main',
        'type' => 'project',
        'install_path' => __DIR__ . '/../../',
        'aliases' => array(),
        'reference' => '2b056ffa5b22c875d59d33572e4bdaa529de53c5',
        'name' => 'icinga/icinga-php-thirdparty',
        'dev' => true,
    ),
    'versions' => array(
        'clue/block-react' => array(
            'pretty_version' => 'v1.5.0',
            'version' => '1.5.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/block-react',
            'aliases' => array(),
            'reference' => '718b0571a94aa693c6fffc72182e87257ac900f3',
            'dev_requirement' => false,
        ),
        'clue/buzz-react' => array(
            'pretty_version' => 'v2.9.0',
            'version' => '2.9.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/buzz-react',
            'aliases' => array(),
            'reference' => 'eb180b5e0db00a3febaa458289706b8ca80ffe2a',
            'dev_requirement' => false,
        ),
        'clue/connection-manager-extra' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/connection-manager-extra',
            'aliases' => array(),
            'reference' => '95e7033af27a74a793b30dc9f049fbaf93de2c2c',
            'dev_requirement' => false,
        ),
        'clue/http-proxy-react' => array(
            'pretty_version' => 'v1.7.0',
            'version' => '1.7.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/http-proxy-react',
            'aliases' => array(),
            'reference' => '8b2f18cd7bc8b2fb383ac24294d28f340a8cfcb8',
            'dev_requirement' => false,
        ),
        'clue/mq-react' => array(
            'pretty_version' => 'v1.4.0',
            'version' => '1.4.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/mq-react',
            'aliases' => array(),
            'reference' => '3f2ea2c2947522022ba5ff6c52267fa3f2c0f193',
            'dev_requirement' => false,
        ),
        'clue/redis-protocol' => array(
            'pretty_version' => 'v0.3.1',
            'version' => '0.3.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-protocol',
            'aliases' => array(),
            'reference' => '271b8009887209d930f613ad3b9518f94bd6b51c',
            'dev_requirement' => false,
        ),
        'clue/redis-react' => array(
            'pretty_version' => 'v2.6.0',
            'version' => '2.6.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/redis-react',
            'aliases' => array(),
            'reference' => 'f911455f9d7a77dd6f39c22548ddff521544b291',
            'dev_requirement' => false,
        ),
        'clue/soap-react' => array(
            'pretty_version' => 'v1.0.0',
            'version' => '1.0.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/soap-react',
            'aliases' => array(),
            'reference' => '72ec7d5a67ccfbfbc80cc349145c8b14f4a7f393',
            'dev_requirement' => false,
        ),
        'clue/socket-raw' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socket-raw',
            'aliases' => array(),
            'reference' => '91e9f619f6769f931454a9882c21ffd7623d06cb',
            'dev_requirement' => false,
        ),
        'clue/socks-react' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/socks-react',
            'aliases' => array(),
            'reference' => 'f90edf489313301e87292c51d41faf8c21138513',
            'dev_requirement' => false,
        ),
        'clue/stdio-react' => array(
            'pretty_version' => 'v2.6.0',
            'version' => '2.6.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/stdio-react',
            'aliases' => array(),
            'reference' => 'dfa6c378aabdff718202d4e2453f752c38ea3399',
            'dev_requirement' => false,
        ),
        'clue/term-react' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/term-react',
            'aliases' => array(),
            'reference' => 'eb6eb063eda04a714ef89f066586a2c49588f7ca',
            'dev_requirement' => false,
        ),
        'clue/utf8-react' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../clue/utf8-react',
            'aliases' => array(),
            'reference' => '8bc3f8c874cdf642c8f10f9ae93aadb8cd63da96',
            'dev_requirement' => false,
        ),
        'components/jquery' => array(
            'pretty_version' => '3.6.0',
            'version' => '3.6.0.0',
            'type' => 'component',
            'install_path' => __DIR__ . '/../components/jquery',
            'aliases' => array(),
            'reference' => '6cf38ee1fd04b6adf8e7dda161283aa35be818c3',
            'dev_requirement' => false,
        ),
        'cweagans/composer-patches' => array(
            'pretty_version' => '1.7.2',
            'version' => '1.7.2.0',
            'type' => 'composer-plugin',
            'install_path' => __DIR__ . '/../cweagans/composer-patches',
            'aliases' => array(),
            'reference' => 'e9969cfc0796e6dea9b4e52f77f18e1065212871',
            'dev_requirement' => false,
        ),
        'evenement/evenement' => array(
            'pretty_version' => 'v3.0.1',
            'version' => '3.0.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../evenement/evenement',
            'aliases' => array(),
            'reference' => '531bfb9d15f8aa57454f5f0285b18bec903b8fb7',
            'dev_requirement' => false,
        ),
        'fig/http-message-util' => array(
            'pretty_version' => '1.1.5',
            'version' => '1.1.5.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../fig/http-message-util',
            'aliases' => array(),
            'reference' => '9d94dc0154230ac39e5bf89398b324a86f63f765',
            'dev_requirement' => false,
        ),
        'guzzlehttp/guzzle' => array(
            'pretty_version' => '6.5.7',
            'version' => '6.5.7.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/guzzle',
            'aliases' => array(),
            'reference' => '724562fa861e21a4071c652c8a159934e4f05592',
            'dev_requirement' => false,
        ),
        'guzzlehttp/promises' => array(
            'pretty_version' => '1.5.1',
            'version' => '1.5.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/promises',
            'aliases' => array(),
            'reference' => 'fe752aedc9fd8fcca3fe7ad05d419d32998a06da',
            'dev_requirement' => false,
        ),
        'guzzlehttp/psr7' => array(
            'pretty_version' => '1.8.5',
            'version' => '1.8.5.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../guzzlehttp/psr7',
            'aliases' => array(),
            'reference' => '337e3ad8e5716c15f9657bd214d16cc5e69df268',
            'dev_requirement' => false,
        ),
        'icinga/icinga-php-thirdparty' => array(
            'pretty_version' => 'dev-main',
            'version' => 'dev-main',
            'type' => 'project',
            'install_path' => __DIR__ . '/../../',
            'aliases' => array(),
            'reference' => '2b056ffa5b22c875d59d33572e4bdaa529de53c5',
            'dev_requirement' => false,
        ),
        'jfcherng/php-color-output' => array(
            'pretty_version' => '2.0.2',
            'version' => '2.0.2.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-color-output',
            'aliases' => array(),
            'reference' => '2673074597eca9682d2fdfaee39a22418d4cc2f6',
            'dev_requirement' => false,
        ),
        'jfcherng/php-diff' => array(
            'pretty_version' => '6.10.4',
            'version' => '6.10.4.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-diff',
            'aliases' => array(),
            'reference' => 'ccc178b81ac737b0f9a60e7210d26d5ed797e786',
            'dev_requirement' => false,
        ),
        'jfcherng/php-mb-string' => array(
            'pretty_version' => '1.4.6',
            'version' => '1.4.6.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-mb-string',
            'aliases' => array(),
            'reference' => 'f5b438d348e030961e0b86a04ef2319c31ae8146',
            'dev_requirement' => false,
        ),
        'jfcherng/php-sequence-matcher' => array(
            'pretty_version' => '3.2.8',
            'version' => '3.2.8.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../jfcherng/php-sequence-matcher',
            'aliases' => array(),
            'reference' => '410519ca07cd7989f450d6e2aa40975dbd1be048',
            'dev_requirement' => false,
        ),
        'paragonie/random_compat' => array(
            'pretty_version' => 'v9.99.100',
            'version' => '9.99.100.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../paragonie/random_compat',
            'aliases' => array(),
            'reference' => '996434e5492cb4c3edcb9168db6fbb1359ef965a',
            'dev_requirement' => false,
        ),
        'predis/predis' => array(
            'pretty_version' => 'v1.1.10',
            'version' => '1.1.10.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../predis/predis',
            'aliases' => array(),
            'reference' => 'a2fb02d738bedadcffdbb07efa3a5e7bd57f8d6e',
            'dev_requirement' => false,
        ),
        'psr/http-message' => array(
            'pretty_version' => '1.0.1',
            'version' => '1.0.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../psr/http-message',
            'aliases' => array(),
            'reference' => 'f6561bf28d520154e4b0ec72be95418abe6d9363',
            'dev_requirement' => false,
        ),
        'psr/http-message-implementation' => array(
            'dev_requirement' => false,
            'provided' => array(
                0 => '1.0',
            ),
        ),
        'ralouphie/getallheaders' => array(
            'pretty_version' => '3.0.3',
            'version' => '3.0.3.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ralouphie/getallheaders',
            'aliases' => array(),
            'reference' => '120b605dfeb996808c31b6477290a714d356e822',
            'dev_requirement' => false,
        ),
        'ramsey/uuid' => array(
            'pretty_version' => '3.9.6',
            'version' => '3.9.6.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ramsey/uuid',
            'aliases' => array(),
            'reference' => 'ffa80ab953edd85d5b6c004f96181a538aad35a3',
            'dev_requirement' => false,
        ),
        'react/cache' => array(
            'pretty_version' => 'v1.1.1',
            'version' => '1.1.1.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/cache',
            'aliases' => array(),
            'reference' => '4bf736a2cccec7298bdf745db77585966fc2ca7e',
            'dev_requirement' => false,
        ),
        'react/child-process' => array(
            'pretty_version' => 'v0.6.4',
            'version' => '0.6.4.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/child-process',
            'aliases' => array(),
            'reference' => 'a778f3fb828d68caf8a9ab6567fd8342a86f12fe',
            'dev_requirement' => false,
        ),
        'react/datagram' => array(
            'pretty_version' => 'v1.8.0',
            'version' => '1.8.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/datagram',
            'aliases' => array(),
            'reference' => '40f2c39f83541367018605a73b57c99186f7145c',
            'dev_requirement' => false,
        ),
        'react/dns' => array(
            'pretty_version' => 'v1.9.0',
            'version' => '1.9.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/dns',
            'aliases' => array(),
            'reference' => '6d38296756fa644e6cb1bfe95eff0f9a4ed6edcb',
            'dev_requirement' => false,
        ),
        'react/event-loop' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/event-loop',
            'aliases' => array(),
            'reference' => '187fb56f46d424afb6ec4ad089269c72eec2e137',
            'dev_requirement' => false,
        ),
        'react/http' => array(
            'pretty_version' => 'v1.6.0',
            'version' => '1.6.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/http',
            'aliases' => array(),
            'reference' => '59961cc4a5b14481728f07c591546be18fa3a5c7',
            'dev_requirement' => false,
        ),
        'react/http-client' => array(
            'pretty_version' => 'v0.5.11',
            'version' => '0.5.11.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/http-client',
            'aliases' => array(),
            'reference' => '23dddb415b9bd36c81d1c78df63143e65702aa4b',
            'dev_requirement' => false,
        ),
        'react/promise' => array(
            'pretty_version' => 'v2.9.0',
            'version' => '2.9.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise',
            'aliases' => array(),
            'reference' => '234f8fd1023c9158e2314fa9d7d0e6a83db42910',
            'dev_requirement' => false,
        ),
        'react/promise-stream' => array(
            'pretty_version' => 'v1.3.0',
            'version' => '1.3.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-stream',
            'aliases' => array(),
            'reference' => '3ebd94fe0d8edbf44937948af28d02d5437e9949',
            'dev_requirement' => false,
        ),
        'react/promise-timer' => array(
            'pretty_version' => 'v1.9.0',
            'version' => '1.9.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/promise-timer',
            'aliases' => array(),
            'reference' => 'aa7a73c74b8d8c0f622f5982ff7b0351bc29e495',
            'dev_requirement' => false,
        ),
        'react/socket' => array(
            'pretty_version' => 'v1.11.0',
            'version' => '1.11.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/socket',
            'aliases' => array(),
            'reference' => 'f474156aaab4f09041144fa8b57c7d70aed32a1c',
            'dev_requirement' => false,
        ),
        'react/stream' => array(
            'pretty_version' => 'v1.2.0',
            'version' => '1.2.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../react/stream',
            'aliases' => array(),
            'reference' => '7a423506ee1903e89f1e08ec5f0ed430ff784ae9',
            'dev_requirement' => false,
        ),
        'rhumsaa/uuid' => array(
            'dev_requirement' => false,
            'replaced' => array(
                0 => '3.9.6',
            ),
        ),
        'ringcentral/psr7' => array(
            'pretty_version' => '1.3.0',
            'version' => '1.3.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../ringcentral/psr7',
            'aliases' => array(),
            'reference' => '360faaec4b563958b673fb52bbe94e37f14bc686',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-ctype' => array(
            'pretty_version' => 'v1.26.0',
            'version' => '1.26.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-ctype',
            'aliases' => array(),
            'reference' => '6fd1b9a79f6e3cf65f9e679b23af304cd9e010d4',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-idn' => array(
            'pretty_version' => 'v1.26.0',
            'version' => '1.26.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-idn',
            'aliases' => array(),
            'reference' => '59a8d271f00dd0e4c2e518104cc7963f655a1aa8',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-intl-normalizer' => array(
            'pretty_version' => 'v1.26.0',
            'version' => '1.26.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-intl-normalizer',
            'aliases' => array(),
            'reference' => '219aa369ceff116e673852dce47c3a41794c14bd',
            'dev_requirement' => false,
        ),
        'symfony/polyfill-php72' => array(
            'pretty_version' => 'v1.26.0',
            'version' => '1.26.0.0',
            'type' => 'library',
            'install_path' => __DIR__ . '/../symfony/polyfill-php72',
            'aliases' => array(),
            'reference' => 'bf44a9fd41feaac72b074de600314a93e2ae78e2',
            'dev_requirement' => false,
        ),
    ),
);
